/* sort driver - use with any program from chap 6 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "sort.h"

int *load_file(int *m);
int *load_random(int m);
int check_sort(Item tbl[], int maxent);
void save_file(Item tbl[], int maxent);
void print_tbl(Item tbl[], int maxent);

main(int argc, char *argv[])
{
	int ret, maxent;
   Item *tbl;
   char buf[BUFSIZ];
   time_t t1, t2, t3;	/* timing variables */

   printf("Sort file or random numbers? (f r): ");
   gets(buf);

   if (*buf == 'f')
   { /* sort from file */
   	tbl = load_file(&maxent);
   }
   else
   { /* sort random numbers */
   	printf("How many numbers: ");
      gets(buf);
      maxent = atoi(buf);
   	tbl = load_random(maxent);
      printf("Save to file? (y or n): ");
      gets(buf);
      if (*buf == 'y')
      {
      	save_file(tbl, maxent);
      }

   }

   t1 = time(NULL);	/* start time */
   ret = sort(tbl, 0, maxent-1);
   t2 = time(NULL);	/* end time */
   t3 = t2 - t1;
   printf("t1=%ld, t2=%ld, t3=%ld\n", t1, t2, t3);

   printf("Sort performed %d swaps, time=%ld\n", ret, t3);

   if (check_sort(tbl, maxent)) printf("Table Sorted Correctly\n");
   else printf("**** Table Not Sorted! ****\n");

   printf("Press ENTER to see results: ");
   gets(buf);

   /* print sort results unless user says no */
   if (*buf != 'n') print_tbl(tbl, maxent);

   return 0;
}

int *
load_file(int *m)
{ /* load array with integers */
	int *ret, num, maxent=1000, nent=0;
   char buf[BUFSIZ];
   FILE *fp;

   printf("Enter name of file: ");
   gets(buf);
   if ((fp = fopen(buf, "r")) == NULL)
   { /* bad file name */
   	printf("Can't open file (%s)\n", buf);
      exit(1);
   }

   /* allocate initial array */
	ret = (int *)malloc(maxent *sizeof(int));

   while (fgets(buf, sizeof(buf), fp) != NULL)
   { /* read lines from file */
   	if (nent >= maxent)
      { /* table full - resize */
      	maxent += MAXENT;	/* increase size */
         ret = realloc(ret, maxent);
      }
   	num = atoi(buf);	/* convert to integer */
      ret[nent++] = num;
   }
   fclose(fp);	/* close file */

   /* file read & stored - wrap up & return */
   *m = nent;	/* return how many entries */
   return ret;	/* return actual array */
}

int *
load_random(int m)
{ /* allocate array of 'm' random numbers */
	int *ret, nent=0;

   /* allocate array to be returned */
   ret = (int *)malloc(m * sizeof(int));

   for (nent=0; nent < m; nent++)
   {
   	ret[nent] = rand() % MAXNUM;	/* max size */
   }

   return ret;
}

void
save_file(Item tbl[], int maxent)
{
	FILE *fp;
   char buf[BUFSIZ];
   int i;

   printf("Enter output file name: ");
   gets(buf);
   if ((fp = fopen(buf, "w")) == NULL)
   {
   	printf("Can't open output file (%s)\n", buf);
      exit(2);
   }

   for (i=0; i < maxent; i++)
   {
   	fprintf(fp, "%d\n", tbl[i]);
   }
   fclose(fp);
}

void
print_tbl(Item tbl[], int maxent)
{
	int i;
   char buf[BUFSIZ];

   for (i=0; i < maxent; i++)
   {
   	printf("%4d %6d\n", i, tbl[i]);
      if (i%25 == 24)
      {
      	printf("Press ENTER to continue, 'q' to quit: ");
         gets(buf);
         if (*buf == 'q') break;
      }
   }
}

int
check_sort(Item tbl[],  int maxent)
{
	int i;

   for (i=1; i < maxent; i++)
   { /* compare entry with prev */
   	if (tbl[i-1] > tbl[i]) return 0;
   }
   return 1;
}

