/* quicksort - sort from chap 6, plust progs 7.1 & 7.2 */

#include <stdio.h>
#include <stdlib.h>

#include "sort.h"

int partition(Item a[], int l, int r);
void quicksort(Item a[], int l, int r);

int *load_file(int *m);
int *load_random(int m);
int check_sort(Item tbl[], int maxent);
void save_file(Item tbl[], int maxent);
void print_tbl(Item tbl[], int maxent);

main(int argc, char *argv[])
{
	int ret, maxent;
   Item *tbl;
   char buf[BUFSIZ];

   printf("Sort file or random numbers? (f r): ");
   gets(buf);

   if (*buf == 'f')
   { /* sort from file */
   	tbl = load_file(&maxent);
   }
   else
   { /* sort random numbers */
   	printf("How many numbers: ");
      gets(buf);
      maxent = atoi(buf);
   	tbl = load_random(maxent);
      printf("Save to file? (y or n): ");
      gets(buf);
      if (*buf == 'y')
      {
      	save_file(tbl, maxent);
      }

   }

   quicksort(tbl, 0, maxent-1);

   if (check_sort(tbl, maxent)) printf("Table Sorted Correctly\n");
   else printf("**** Table Not Sorted Correctly! ****\n");

   printf("Press ENTER to see results: ");
   gets(buf);

   /* print sort results unless user says no */
   if (*buf != 'n') print_tbl(tbl, maxent);

   printf("Press ENTER to terminate program: ");
   gets(buf);

   return 0;
}

int *
load_file(int *m)
{ /* load array with integers */
	int *ret, num, maxent=1000, nent=0;
   char buf[BUFSIZ];
   FILE *fp;

   printf("Enter name of file: ");
   gets(buf);
   if ((fp = fopen(buf, "r")) == NULL)
   { /* bad file name */
   	printf("Can't open file (%s)\n", buf);
      exit(1);
   }

   /* allocate initial array */
	ret = (int *)malloc(maxent *sizeof(int));

   while (fgets(buf, sizeof(buf), fp) != NULL)
   { /* read lines from file */
   	if (nent >= maxent)
      { /* table full - resize */
      	maxent += MAXENT;	/* increase size */
         ret = realloc(ret, maxent);
      }
   	num = atoi(buf);	/* convert to integer */
      ret[nent++] = num;
   }
   fclose(fp);	/* close file */

   /* file read & stored - wrap up & return */
   *m = nent;	/* return how many entries */
   return ret;	/* return actual array */
}

int *
load_random(int m)
{ /* allocate array of 'm' random numbers */
	int *ret, nent=0;

   /* allocate array to be returned */
   ret = (int *)malloc(m * sizeof(int));

   for (nent=0; nent < m; nent++)
   {
   	ret[nent] = rand() % MAXNUM;	/* max size */
   }

   return ret;
}

void
save_file(Item tbl[], int maxent)
{
	FILE *fp;
   char buf[BUFSIZ];
   int i;

   printf("Enter output file name: ");
   gets(buf);
   if ((fp = fopen(buf, "w")) == NULL)
   {
   	printf("Can't open output file (%s)\n", buf);
      exit(2);
   }

   for (i=0; i < maxent; i++)
   {
   	fprintf(fp, "%d\n", tbl[i]);
   }
   fclose(fp);
}

void
print_tbl(Item tbl[], int maxent)
{
	int i;
   char buf[BUFSIZ];

   for (i=0; i < maxent; i++)
   {
   	printf("%4d %6d\n", i, tbl[i]);
      if (i%25 == 24)
      {
      	printf("Press ENTER to continue, 'q' to quit: ");
         gets(buf);
         if (*buf == 'q') break;
      }
   }
}

int
check_sort(Item tbl[],  int maxent)
{
	int i;

   for (i=1; i < maxent; i++)
   { /* compare entry with prev */
   	if (tbl[i-1] > tbl[i]) return 0;
   }
   return 1;
}

void quicksort(Item a[], int l, int r)
{
	int i, ret=0;

   /* first, see if we are already done */
   if (r <= l) return ret;

   /* partition array segment into two segments */
   i = partition(a, l, r);

   /* now, recursively sort each partition */
   quicksort(a, l, i-1);
   quicksort(a, i+1, r);
}

int partition(Item a[], int l, int r)
{
	int i, j;	/* left & right scan */
   Item v;		/* item to partition segment */

   i = l-1;
   j = r;
   v = a[r];

   /*
    * scan left & right to partition segment
    * find first element from left that is bigger than v
    * find first element from right that is smaller than v
    * if left > right, we are done
    * otherwise, swap the left & right elements
    */
   while (1)
   {
   	/* find first element from left that is not less than v */
   	while (less(a[++i], v)) ;

      /* find first element from right that is not bigger than v */
      while (less(v, a[--j]))
      { /* if we reach the left end, we are done */
	      if (j == l) break;
      }

      /* if the two scans crossed, we are done */
      if (i >= j) break;

      /*
       * we have found two elements that violate
       * the criteria:
       *		a[i] is left of center and bigger than v,
       *		a[j] is right of center and smaller than v
       * swap a[i] & a[j] to enforce the criteria
       */
      exch(a[i], a[j]);
   }

   /*
    * everything left of a[i] is smaller than v
    * everything right of a[i] is bigger than v
    * swap v into a[i]
    */
   exch(a[i], a[r]);

   /* return new partition location */
   return i;
}

